@@ -1,19 +1,36 @@
 """Top-level package for lingtreemaps."""
 import logging
 import sys
+import typing
+import warnings
 from io import StringIO
+from pathlib import Path
+import Bio.Phylo.Newick
 import colorlog
 import contextily as cx
 import geopandas as gpd
+import matplotlib
 import matplotlib.pyplot as plt
 import pandas as pd
 import requests
 import seaborn as sns
+import shapely
 import shapely.geometry
 import yaml
 from Bio import Phylo
 from matplotlib import patheffects
 from matplotlib.patches import Patch
+from lingtreemaps.helpers import read_data_file
+
+
+try:
+    from shapely.errors import ShapelyDeprecationWarning
+
+    warnings.filterwarnings("ignore", category=ShapelyDeprecationWarning)
+except ImportError:
+    pass
+
+__all__ = ["plot", "get_glottolog_csv", "download_glottolog_tree", "load_conf"]
 
 
 handler = colorlog.StreamHandler(None)
@@ -27,7 +44,7 @@ log.addHandler(handler)
 
 __author__ = "Florian Matter"
 __email__ = "florianmatter@gmail.com"
-__version__ = "0.0.2"
+__version__ = "0.0.5"
 
 
 try:
@@ -36,6 +53,20 @@ except ImportError:  # pragma: no cover
     from importlib_resources import files  # pragma: no cover
 
 data_path = files("lingtreemaps") / "data"
+plt.rcParams.update({"hatch.color": "white"})
+
+
+def load_conf(conf_file: typing.Union[Path, str] = "lingtreemaps.yaml") -> dict:
+    confpath = Path(conf_file)
+    if confpath.is_file():
+        with open(confpath, "r", encoding="utf-8") as f:
+            data = yaml.load(f, yaml.SafeLoader)
+            if data is not None:
+                return data
+            log.warning(f"Empty config file: {confpath.resolve()}")
+            return {}
+    log.error(f"Config file {confpath.resolve()} not found.")
+    return {}
 
 
 def download_glottolog_tree(root, df=None):
@@ -83,9 +114,17 @@ def get_glottolog_csv(glottocode):
     return df
 
 
-def plot(lg_df, tree, feature_df=None, text_df=None, **kwargs):
+def plot(
+    lg_df: pd.DataFrame,
+    tree: Bio.Phylo.Newick.Tree,
+    feature_df: typing.Optional[pd.DataFrame] = None,
+    text_df: typing.Optional[pd.DataFrame] = None,
+    **kwargs,
+):
     with open(data_path / "default_config.yaml", "r", encoding="utf-8") as f:
         conf = yaml.load(f, Loader=yaml.SafeLoader)
+    if isinstance(text_df, str):
+        text_df = read_data_file(text_df, keep_default_na=False)
     conf.update(**kwargs)
     plot_map(lg_df, tree, feature_df, text_df, **conf)
 
@@ -111,6 +150,7 @@ def plot_map(  # noqa: MC0001
     internal_map_padding,
     seaborn_palette,
     color_dict,
+    hatch_dict,
     legend_position,
     leaf_marker_size,
     leaf_lw,
@@ -122,6 +162,7 @@ def plot_map(  # noqa: MC0001
     print_labels,
     nonterminal_nodes,
     color_tree,
+    hatching,
     text_x_offset,
     text_y_offset,
     base_padding,
@@ -139,16 +180,20 @@ def plot_map(  # noqa: MC0001
     default_color = "orange"
     if feature_df is not None:
         if "Clade" not in feature_df.columns or "Value" not in feature_df.columns:
-            raise ValueError("Feature dataframe has to have columns 'Clade' and 'Value'")
+            raise ValueError(
+                "Feature dataframe has to have columns 'Clade' and 'Value'"
+            )
         lg_df = pd.merge(
             lg_df, feature_df, left_on=id_col, right_on="Clade", how="outer"
         )  # insert feature values into language table
-        lg_df[id_col] = lg_df.apply(lambda x: x["Clade"] if pd.isnull(x[id_col]) else x[id_col], axis=1)
+        lg_df[id_col] = lg_df.apply(
+            lambda x: x["Clade"] if pd.isnull(x[id_col]) else x[id_col], axis=1
+        )
+        values = []
+        for x in list(feature_df["Value"]):
+            if x not in values:
+                values.append(x)
         if color_dict is None:
-            values = []
-            for x in list(feature_df["Value"]):
-                if x not in values:
-                    values.append(x)
             palette = sns.color_palette(
                 seaborn_palette, len(values)
             )  # generate palette
@@ -157,12 +202,34 @@ def plot_map(  # noqa: MC0001
             )  # what value corresponds to what color?
         else:
             value_color_dic = color_dict
+        if hatching and hatch_dict is None:
+            value_hatch_dic = dict(
+                zip(
+                    values,
+                    [
+                        "///",
+                        "\\\\\\",
+                        "|||",
+                        "---",
+                        "+++",
+                        "xxx",
+                        "ooo",
+                        "O00",
+                        "...",
+                        "***",
+                    ],
+                )
+            )
+            hatch_dict = {}
+        else:
+            value_hatch_dic = hatch_dict
         lg_df["color"] = lg_df["Value"].apply(
             lambda x: value_color_dic.get(x, (0, 0, 0, 0))
         )  # use transparent color for missing values
         color_dic = dict(
             zip(lg_df[id_col], lg_df["color"])
         )  # what (leaf) name corresponds to what color?
+
     else:
         lg_df["color"] = default_color
         color_dic = {}
@@ -172,6 +239,27 @@ def plot_map(  # noqa: MC0001
         lg_df, geometry=gpd.points_from_xy(lg_df.Longitude, lg_df.Latitude)
     )  # convert to a geodataframe
 
+    # Hack for cases where a value only occurs once
+    if hatching:
+        fake_list = []
+        for value, hatch in value_hatch_dic.items():
+            fake_list.append(
+                {
+                    "ID": f"dummy_{value}",
+                    "color": value_color_dic[value],
+                    "Value": value,
+                    "Latitude": -83.747128,
+                    "Longitude": 2.431754,
+                }
+            )
+        lg_df = pd.concat([lg_df, pd.DataFrame.from_dict(fake_list)])
+
+        fake_gdf = gpd.GeoDataFrame(
+            lg_df, geometry=gpd.points_from_xy(lg_df.Longitude, lg_df.Latitude)
+        )  # convert to a geodataframe
+    else:
+        fake_gdf = gdf.copy()
+
     gdf.dropna(inplace=True, subset=["Latitude", "Longitude"])
 
     bounds = gdf.geometry.total_bounds  # outer boundaries of the language points
@@ -244,7 +332,6 @@ def plot_map(  # noqa: MC0001
     leaf_count = len(tree.get_terminals())
     map_height = abs(gdf.geometry.total_bounds[1] - gdf.geometry.total_bounds[3])
     leaf_spacing = map_height / leaf_count
-    leaf_marker_size = leaf_marker_size or leaf_spacing * 0.3
 
     bounds = gdf.geometry.total_bounds  # tight box around the points
     internal_map_padding = internal_map_padding or leaf_spacing * 0.5
@@ -268,9 +355,6 @@ def plot_map(  # noqa: MC0001
     ]  # larger padding box
     visible_map_rect = shapely.geometry.box(*visible_map)
 
-    text_x_offset = text_x_offset or leaf_marker_size * 1.3
-    text_y_offset = text_y_offset or leaf_marker_size * 0.4
-
     base_padding = base_padding or tree_depth * 0.05
 
     if debug:
@@ -304,14 +388,35 @@ def plot_map(  # noqa: MC0001
     else:
         ax.axis("off")
 
-    gdf.plot(
-        ax=ax,
-        markersize=map_marker_size,
-        facecolor=gdf["color"],
-        linewidth=map_marker_lw,
-        edgecolor="black",
-        zorder=99,
-    )
+    leaf_df = fake_gdf.copy()
+
+    if hatching:
+        for value, hatch in hatch_dict.items():
+            fake_gdf[fake_gdf["Value"] == value].plot(
+                ax=ax,
+                markersize=map_marker_size,
+                facecolor=fake_gdf[fake_gdf["Value"] == value]["color"],
+                linewidth=map_marker_lw,
+                zorder=99,
+                hatch=hatch,
+            )
+        gdf[pd.isnull(gdf["Value"])].plot(
+            ax=ax,
+            markersize=map_marker_size,
+            facecolor=gdf[pd.isnull(gdf["Value"])]["color"],
+            linewidth=map_marker_lw,
+            edgecolor="black",
+            zorder=88,
+        )
+    else:
+        gdf.plot(
+            ax=ax,
+            markersize=map_marker_size,
+            facecolor=gdf["color"],
+            linewidth=map_marker_lw,
+            edgecolor="black",
+            zorder=99,
+        )
 
     leaf_positions = {}
     i = 0
@@ -326,6 +431,12 @@ def plot_map(  # noqa: MC0001
     tree_baseline = visible_map[0] - tree_map_padding
     sideline = bounds[-1] - leaf_spacing * 0.5
 
+    node_leafs = {}
+
+    if hatching:
+        for value, hatch in hatch_dict.items():
+            node_leafs["dummy_" + value] = (-83.747128, 2.431754)
+
     def draw_clade(clade, color, lw):
         lvl_line_start = get_clade_middle(clade.clades[0])
         lvl_line_end = get_clade_middle(clade.clades[-1])
@@ -380,6 +491,7 @@ def plot_map(  # noqa: MC0001
                         tree_baseline - child_depth,
                         sideline - get_clade_middle(child),
                     )
+                    node_leafs[child.name] = leaf_coords
                     node_alpha = 1
                     for point in gdf[gdf[id_col] == child.name].to_dict("records"):
                         map_node = (point["geometry"].x, point["geometry"].y)
@@ -411,15 +523,6 @@ def plot_map(  # noqa: MC0001
                         )
                         if node_alpha == 1:
                             node_alpha = 0
-                    circle = plt.Circle(
-                        leaf_coords,
-                        leaf_marker_size,
-                        facecolor=color_dic.get(child.name, default_color),
-                        edgecolor="black",
-                        zorder=99,
-                        lw=leaf_lw,
-                    )
-                    ax.add_patch(circle)
                 else:
                     if nonterminal_nodes and child.name in color_dic:
                         circle = plt.Circle(
@@ -439,10 +542,23 @@ def plot_map(  # noqa: MC0001
 
     if feature_df is not None:
         # https://www.python-graph-gallery.com/how-to-use-rectangles-in-matplotlib-legends
-        handles = [
-            Patch(facecolor=color, label=label)
-            for label, color in value_color_dic.items()
-        ]
+        if hatching:
+            handles = [
+                matplotlib.patches.Circle(
+                    (0.5, 0.5),
+                    1,
+                    label=label,
+                    facecolor=value_color_dic.get(label, default_color),
+                    linewidth=3,
+                    hatch=value_hatch_dic.get(label, None),
+                )
+                for label, color in value_color_dic.items()
+            ]
+        else:
+            handles = [
+                Patch(facecolor=color, label=label)
+                for label, color in value_color_dic.items()
+            ]
 
         bbox_coords = (
             visible_map[0],
@@ -482,6 +598,38 @@ def plot_map(  # noqa: MC0001
     tree_color = color_dic.get(tree.root.name, "black")
     draw_clade(tree.root, tree_color, tree_lw)
 
+    leaf_df["geometry"] = leaf_df.apply(
+        lambda x: shapely.geometry.Point(node_leafs[x[id_col]]), axis=1
+    )
+
+    if hatching:
+        for value, hatch in hatch_dict.items():
+            leaf_df[leaf_df["Value"] == value].plot(
+                ax=ax,
+                markersize=leaf_marker_size,
+                facecolor=leaf_df[leaf_df["Value"] == value]["color"],
+                linewidth=leaf_lw,
+                zorder=99,
+                hatch=hatch,
+            )
+        leaf_df[pd.isnull(leaf_df["Value"])].plot(
+            ax=ax,
+            markersize=leaf_marker_size,
+            facecolor=leaf_df[pd.isnull(leaf_df["Value"])]["color"],
+            linewidth=leaf_lw,
+            edgecolor="black",
+            zorder=99,
+        )
+    else:
+        leaf_df.plot(
+            ax=ax,
+            markersize=leaf_marker_size,
+            facecolor=leaf_df["color"],
+            linewidth=leaf_lw,
+            edgecolor="black",
+            zorder=99,
+        )
+
     def get_attribution_position(position):
         if position == "bottomleft":
             return (
